﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Globalization;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Cluster.Clunet;
using MovablePython;
using System.IO;
using Cluster.WinLIRC;
using System.Threading;
//using Cluster.SpeechToText;
using System.Text.RegularExpressions;
using Microsoft.Win32;

namespace Cluster.Clunet.Console
{
    public partial class ClunetConsoleForm : Form
    {
        public const byte LocalAddress = 0x00;
        readonly ClunetConnection clunetConnection;
        private Dictionary<byte, string> deviceNames = new Dictionary<byte, string>();
        private RcControlForm philipsForm, yamahaForm, engyForm, laserForm;
        private DateTime lastPress = DateTime.Now;
        private Dictionary<string, double> temperature = new Dictionary<string, double>();
        private WinLircServer winLircServer;
        private int[] lastWheelInfo = new int[2];
        private int noMotionTime = 0;
        //private SpeechCapture speechCapture = new SpeechCapture();

        public ClunetConsoleForm(ClunetConnection clunetConnection)
        {
            InitializeComponent();
            this.clunetConnection = clunetConnection;
            clunetConnection.OnConnected += clunetConnection_OnConnected;
            clunetConnection.OnDisconnected += clunetConnection_OnConnected;
            clunetConnection.OnPacketReceived += clunetConnection_OnPacketReceived;
            UpdateConnectionStatus();
            deviceNames[0] = "CLUNET gateway";
            comboBoxPriority.SelectedIndex = 3;
            comboBoxCommands.Items.Clear();
            byte i = 0;
            while (true)
            {
                string commandName;
                var command = (ClunetPacket.CommandType)i;
                commandName = command.ToString();
                if (commandName != i.ToString())
                    comboBoxCommands.Items.Add(string.Format("{0:X2} - {1}", i, commandName));
                i++;
                if (i == 0) break;
            }
            comboBoxCommands.SelectedIndex = 0;
            comboBoxDevices.SelectedIndex = 0;
            yamahaForm = new RcControlForm(clunetConnection,
                Image.FromFile(Path.Combine(Path.GetDirectoryName(Application.ExecutablePath), "yamaha_big.jpg")),
                Path.Combine(Path.GetDirectoryName(Application.ExecutablePath), "yamaha.dat"), 0.85);
            //yamahaForm.ScanImage(Image.FromFile(Path.Combine(Path.GetDirectoryName(Application.ExecutablePath), "yamaha_mask_big.png")));
            philipsForm = new RcControlForm(clunetConnection,
                Image.FromFile(Path.Combine(Path.GetDirectoryName(Application.ExecutablePath), "philips_big.jpg")),
                Path.Combine(Path.GetDirectoryName(Application.ExecutablePath), "philips.dat"), 0.5);
            engyForm = new RcControlForm(clunetConnection,
                Image.FromFile(Path.Combine(Path.GetDirectoryName(Application.ExecutablePath), "engy.jpg")),
                Path.Combine(Path.GetDirectoryName(Application.ExecutablePath), "engy.dat"), 1);
            laserForm = new RcControlForm(clunetConnection,
                Image.FromFile(Path.Combine(Path.GetDirectoryName(Application.ExecutablePath), "laser.jpg")),
                Path.Combine(Path.GetDirectoryName(Application.ExecutablePath), "laser.dat"), 1);
            //engyForm.ScanImage(Image.FromFile(Path.Combine(Path.GetDirectoryName(Application.ExecutablePath), "engy_mask.png")));
            //philipsForm.ScanImage(Image.FromFile(Path.Combine(Path.GetDirectoryName(Application.ExecutablePath), "philips_mask_big.png")));
            //laserForm.ScanImage(Image.FromFile(Path.Combine(Path.GetDirectoryName(Application.ExecutablePath), "laser_mask.png")));
            //yamahaForm.ExportToHtml(@"D:\yamaha.html", "yamaha.jpg", "/rcontrol/", 1);
            //yamahaForm.ExportToJava(@"D:\temp\jamaha.java", 1);            
            //philipsForm.ExportToHtml(@"D:\philips.html", "philips.jpg", "/rcontrol/", 1);
            //philipsForm.ExportToJava(@"D:\temp\philips.java", 1);
            //timberkForm.ExportToJava(@"D:\temp\timberk.java", 1);
            //laserForm.ExportToJava(@"D:\temp\laser.java", 1);
            //laserForm.ExportToHtml(@"D:\Temp\laser.html", "laser.jpg", "/rcontrol", 1);
            //timberkForm.ExportToHtml(@"D:\Temp\timberk.html", "timberk.jpg", "/rcontrol.php", 1);
            //engyForm.ExportToHtml(@"D:\Temp\engy.html", "engy.jpg", "/rcontrol.php", 1);
            //engyForm.ExportToJava(@"D:\temp\engy.java", 1);

            Hotkey hkTvRemote = new Hotkey();
            hkTvRemote.KeyCode = Keys.F1;
            hkTvRemote.Control = hkTvRemote.Alt = true;
            hkTvRemote.Pressed += hkTvRemote_Pressed;
            hkTvRemote.Register(philipsForm);

            Hotkey hkYamahaRemote = new Hotkey();
            hkYamahaRemote.KeyCode = Keys.F2;
            hkYamahaRemote.Control = hkYamahaRemote.Alt = true;
            hkYamahaRemote.Pressed += hkReceiverRemote_Pressed;
            hkYamahaRemote.Register(yamahaForm);

            Hotkey hkLaserRemote = new Hotkey();
            hkLaserRemote.KeyCode = Keys.F3;
            hkLaserRemote.Control = hkLaserRemote.Alt = true;
            hkLaserRemote.Pressed += hkLaserRemote_Pressed;
            hkLaserRemote.Register(engyForm);

            Hotkey hkTimberkRemote = new Hotkey();
            hkTimberkRemote.KeyCode = Keys.F4;
            hkTimberkRemote.Control = hkTimberkRemote.Alt = true;
            hkTimberkRemote.Pressed += hkEngyRemote_Pressed;
            hkTimberkRemote.Register(engyForm);

            Hotkey hkVolumeMinus = new Hotkey();
            hkVolumeMinus.KeyCode = Keys.Down;
            hkVolumeMinus.Control = hkVolumeMinus.Alt = true;
            hkVolumeMinus.Pressed += hkVolumeMinus_Pressed;
            hkVolumeMinus.Register(this);

            Hotkey hkVolumePlus = new Hotkey();
            hkVolumePlus.KeyCode = Keys.Up;
            hkVolumePlus.Control = hkVolumePlus.Alt = true;
            hkVolumePlus.Pressed += hkVolumePlus_Pressed;
            hkVolumePlus.Register(this);

            Hotkey hkVolumeMinusFast = new Hotkey();
            hkVolumeMinusFast.KeyCode = Keys.Down;
            hkVolumeMinusFast.Control = hkVolumeMinusFast.Alt = hkVolumeMinusFast.Shift = true;
            hkVolumeMinusFast.Pressed += hkVolumeMinusFast_Pressed;
            hkVolumeMinusFast.Register(this);

            Hotkey hkVolumePlusFast = new Hotkey();
            hkVolumePlusFast.KeyCode = Keys.Up;
            hkVolumePlusFast.Control = hkVolumePlusFast.Alt = hkVolumePlusFast.Shift = true;
            hkVolumePlusFast.Pressed += hkVolumePlusFast_Pressed;
            hkVolumePlusFast.Register(this);

            Hotkey hkLightMinus = new Hotkey();
            hkLightMinus.KeyCode = Keys.F11;
            hkLightMinus.Control = hkLightMinus.Alt = true;
            hkLightMinus.Pressed += hkLightMinus_Pressed;
            hkLightMinus.Register(this);

            Hotkey hkLightPlus = new Hotkey();
            hkLightPlus.KeyCode = Keys.F12;
            hkLightPlus.Control = hkLightPlus.Alt = true;
            hkLightPlus.Pressed += hkLightPlus_Pressed;
            hkLightPlus.Register(this);

            Hotkey hkDoorLockOpen = new Hotkey();
            hkDoorLockOpen.KeyCode = Keys.F9;
            hkDoorLockOpen.Control = hkDoorLockOpen.Alt = true;
            hkDoorLockOpen.Pressed += hkDoorLockOpen_Pressed;
            hkDoorLockOpen.Register(this);

            Hotkey hkDoorLockClose = new Hotkey();
            hkDoorLockClose.KeyCode = Keys.F10;
            hkDoorLockClose.Control = hkDoorLockClose.Alt = true;
            hkDoorLockClose.Pressed += hkDoorLockClose_Pressed;
            hkDoorLockClose.Register(this);

            Hotkey hkWindowOpenBit = new Hotkey();
            hkWindowOpenBit.KeyCode = Keys.F6;
            hkWindowOpenBit.Control = hkWindowOpenBit.Alt = hkWindowOpenBit.Shift = true;
            hkWindowOpenBit.Pressed += hkWindowOpenBit_Pressed;
            hkWindowOpenBit.Register(this);

            Hotkey hkWindowOpenFull = new Hotkey();
            hkWindowOpenFull.KeyCode = Keys.F6;
            hkWindowOpenFull.Control = hkWindowOpenFull.Alt = true;
            hkWindowOpenFull.Pressed += hkWindowOpenFull_Pressed;
            hkWindowOpenFull.Register(this);

            Hotkey hkWindowOpen = new Hotkey();
            hkWindowOpen.KeyCode = Keys.F7;
            hkWindowOpen.Control = hkWindowOpen.Alt = true;
            hkWindowOpen.Pressed += hkWindowOpen_Pressed;
            hkWindowOpen.Register(this);

            Hotkey hkWindowClose = new Hotkey();
            hkWindowClose.KeyCode = Keys.F8;
            hkWindowClose.Control = hkWindowClose.Alt = true;
            hkWindowClose.Pressed += hkWindowClose_Pressed;
            hkWindowClose.Register(this);

            Hotkey hkWindowCloseBit = new Hotkey();
            hkWindowCloseBit.KeyCode = Keys.F8;
            hkWindowCloseBit.Control = hkWindowCloseBit.Alt = hkWindowCloseBit.Shift = true;
            hkWindowCloseBit.Pressed += hkWindowCloseBit_Pressed;
            hkWindowCloseBit.Register(this);

            winLircServer = new WinLircServer();

            /*
            speechCapture.OnSpeechRecognized += speechCapture_OnSpeechRecognized;
            speechCapture.OnCaptureEror += speechCapture_OnCaptureEror;
            speechCapture.OnSpeechRecognizeFailed += speechCapture_OnSpeechRecognizeFailed;
            speechCapture.StartCapture();
             */

            // Чтобы был свежий IE
            try
            {
                Registry.SetValue(@"HKEY_LOCAL_MACHINE\SOFTWARE\Microsoft\Internet Explorer\MAIN\FeatureControl\FEATURE_BROWSER_EMULATION", Path.GetFileName(Application.ExecutablePath), 0x2af9, RegistryValueKind.DWord);
                Registry.SetValue(@"HKEY_LOCAL_MACHINE\SOFTWARE\Wow6432Node\Microsoft\Internet Explorer\MAIN\FeatureControl\FEATURE_BROWSER_EMULATION", Path.GetFileName(Application.ExecutablePath), 0x2af9, RegistryValueKind.DWord);
            }
            catch { }
        }

        // Посылает команду на увеление громкости с удержанием кнопки, т.е. сразу ~на десяток децибел
        void hkVolumePlusFast_Pressed(object sender, HandledEventArgs e)
        {
            try
            {
                if ((DateTime.Now - lastPress).TotalMilliseconds < 150) return;
                var key = new byte[] { 0x01, 0x7A, 0x85, 0x1A, 0xE5, 0x23 };
                clunetConnection.Send(new ClunetPacket(ClunetPacket.PacketPriority.Command, LocalAddress, 4, ClunetPacket.CommandType.RcButtonSend, key));
                lastPress = DateTime.Now;
            }
            catch { }
        }

        // Посылает команду на уменьшение громкости с удержанием кнопки, т.е. сразу ~на десяток децибел
        void hkVolumeMinusFast_Pressed(object sender, HandledEventArgs e)
        {
            try
            {
                if ((DateTime.Now - lastPress).TotalMilliseconds < 150) return;
                var key = new byte[] { 0x01, 0x7A, 0x85, 0x1B, 0xE4, 0x23 };
                clunetConnection.Send(new ClunetPacket(ClunetPacket.PacketPriority.Command, LocalAddress, 4, ClunetPacket.CommandType.RcButtonSend, key));
                lastPress = DateTime.Now;
            }
            catch { }
        }

        // Прибавление света
        void hkLightPlus_Pressed(object sender, HandledEventArgs e)
        {
            try
            {
                if ((DateTime.Now - lastPress).TotalMilliseconds < 150) return;
                clunetConnection.Send(new ClunetPacket(ClunetPacket.PacketPriority.Command, LocalAddress, 0x01, ClunetPacket.CommandType.DevicePowerCommand, new byte[] { 0xFF }));
                lastPress = DateTime.Now;
            }
            catch { }
        }

        // Убавление света
        void hkLightMinus_Pressed(object sender, HandledEventArgs e)
        {
            try
            {
                if ((DateTime.Now - lastPress).TotalMilliseconds < 150) return;
                var key = new byte[] { 0x01, 0x7A, 0x85, 0x1B, 0xE4 };
                clunetConnection.Send(new ClunetPacket(ClunetPacket.PacketPriority.Command, LocalAddress, 0x01, ClunetPacket.CommandType.DevicePowerCommand, new byte[] { 0xFE }));
                lastPress = DateTime.Now;
            }
            catch { }
        }

        // Посылает команду на увеление громкости 
        void hkVolumePlus_Pressed(object sender, HandledEventArgs e)
        {
            try
            {
                if ((DateTime.Now - lastPress).TotalMilliseconds < 150) return;
                var key = new byte[] { 0x01, 0x7A, 0x85, 0x1A, 0xE5 };
                clunetConnection.Send(new ClunetPacket(ClunetPacket.PacketPriority.Command, LocalAddress, 4, ClunetPacket.CommandType.RcButtonSend, key));
                lastPress = DateTime.Now;
            }
            catch { }
        }

        // Посылает команду на уменьшение громкости 
        void hkVolumeMinus_Pressed(object sender, HandledEventArgs e)
        {
            try
            {
                if ((DateTime.Now - lastPress).TotalMilliseconds < 150) return;
                var key = new byte[] { 0x01, 0x7A, 0x85, 0x1B, 0xE4 };
                clunetConnection.Send(new ClunetPacket(ClunetPacket.PacketPriority.Command, LocalAddress, 4, ClunetPacket.CommandType.RcButtonSend, key));
                lastPress = DateTime.Now;
            }
            catch { }
        }

        void hkDoorLockOpen_Pressed(object sender, HandledEventArgs e)
        {
            try
            {
                clunetConnection.Send(new ClunetPacket(ClunetPacket.PacketPriority.Command, LocalAddress, 0x0A, ClunetPacket.CommandType.DoorLockCommand, new byte[] { 0x01 }));
            }
            catch { }
        }

        void hkDoorLockClose_Pressed(object sender, HandledEventArgs e)
        {
            try
            {
                clunetConnection.Send(new ClunetPacket(ClunetPacket.PacketPriority.Command, LocalAddress, 0x0A, ClunetPacket.CommandType.DoorLockCommand, new byte[] { 0x02 }));
            }
            catch { }
        }

        void hkWindowOpenBit_Pressed(object sender, HandledEventArgs e)
        {
            try
            {
                clunetConnection.Send(new ClunetPacket(ClunetPacket.PacketPriority.Command, LocalAddress, 0x0C, ClunetPacket.CommandType.WindowCommand, new byte[] { 0x03 }));
            }
            catch { }
        }

        void hkWindowOpenFull_Pressed(object sender, HandledEventArgs e)
        {
            try
            {
                clunetConnection.Send(new ClunetPacket(ClunetPacket.PacketPriority.Command, LocalAddress, 0x0C, ClunetPacket.CommandType.WindowCommand, new byte[] { 0x00 }));
            }
            catch { }
        }

        void hkWindowOpen_Pressed(object sender, HandledEventArgs e)
        {
            try
            {
                clunetConnection.Send(new ClunetPacket(ClunetPacket.PacketPriority.Command, LocalAddress, 0x0C, ClunetPacket.CommandType.WindowCommand, new byte[] { 0x01 }));
            }
            catch { }
        }

        void hkWindowClose_Pressed(object sender, HandledEventArgs e)
        {
            try
            {
                clunetConnection.Send(new ClunetPacket(ClunetPacket.PacketPriority.Command, LocalAddress, 0x0C, ClunetPacket.CommandType.WindowCommand, new byte[] { 0x02 }));
            }
            catch { }
        }

        void hkWindowCloseBit_Pressed(object sender, HandledEventArgs e)
        {
            try
            {
                clunetConnection.Send(new ClunetPacket(ClunetPacket.PacketPriority.Command, LocalAddress, 0x0C, ClunetPacket.CommandType.WindowCommand, new byte[] { 0x04 }));
            }
            catch { }
        }

        void hkReceiverRemote_Pressed(object sender, HandledEventArgs e)
        {
            if (philipsForm.Focused)
                philipsForm.Hide();
            else
            {
                philipsForm.Show();
                philipsForm.WindowState = FormWindowState.Normal;
                philipsForm.Activate();
            }
        }

        void hkTvRemote_Pressed(object sender, HandledEventArgs e)
        {
            if (yamahaForm.Focused)
                yamahaForm.Hide();
            else
            {
                yamahaForm.Show();
                yamahaForm.WindowState = FormWindowState.Normal;
                yamahaForm.Activate();
            }
        }

        void hkEngyRemote_Pressed(object sender, HandledEventArgs e)
        {
            if (engyForm.Focused)
                engyForm.Hide();
            else
            {
                engyForm.Show();
                engyForm.WindowState = FormWindowState.Normal;
                engyForm.Activate();
            }
        }

        void hkLaserRemote_Pressed(object sender, HandledEventArgs e)
        {
            if (laserForm.Focused)
                laserForm.Hide();
            else
            {
                laserForm.Show();
                laserForm.WindowState = FormWindowState.Normal;
                laserForm.Activate();
            }
        }

        void clunetConnection_OnConnected()
        {
            try
            {
                StartDiscovery();
                Thread.Sleep(500);
                RequestPowerInfo();
                Thread.Sleep(500);
                RequestIntercomMode();
                Thread.Sleep(500);
                RequestDoorLockState();
                Thread.Sleep(500);
                RequestWindowState();
            }
            catch (Exception) { }
            UpdateConnectionStatus();

            for (int wheel = 0; wheel < lastWheelInfo.Length; wheel++)
                lastWheelInfo[wheel] = 0;
            noMotionTime = 0;
        }

        private delegate void UpdateConnectionStatusDelegate();
        private void UpdateConnectionStatus()
        {
            if (InvokeRequired)
            {
                this.Invoke(new UpdateConnectionStatusDelegate(UpdateConnectionStatus));
                return;
            }
            labelConnectedStatus.Text = (clunetConnection.Connected) ? "Подключены" : "Не подключены";
        }

        private void ClunetConsoleForm_FormClosed(object sender, FormClosedEventArgs e)
        {
            Exit();
        }

        delegate void AddPacketDelegate(ClunetPacket packet);
        private void AddPacketToLog(ClunetPacket packet)
        {
            if (InvokeRequired)
            {
                Invoke(new AddPacketDelegate(AddPacketToLog), new object[] { packet });
                return;
            }
            if (packet.Command == (byte)ClunetPacket.CommandType.Discovery)
            {
                comboBoxDevices.Items.Clear();
                comboBoxDevices.Items.Add("FF - всем");
                comboBoxDevices.SelectedIndex = 0;
                labelTotalDevices.Text = "Всего устройств: 0";
            }
            if (packet.Command == (byte)ClunetPacket.CommandType.DiscoveryResponse)
            {
                comboBoxDevices.Items.Add(string.Format("{0:X2} - {1}", packet.SrcAddress, packet.DataAsString));
                comboBoxDevices.Sorted = true;
                labelTotalDevices.Text = "Всего устройств: " + (comboBoxDevices.Items.Count - 1);
            }
            if (packet.Command == (byte)ClunetPacket.CommandType.WheelInfo)
            {
                lastWheelInfo[packet.Data[0] - 1] = 0;
            }
            if (packet.Command == (byte)ClunetPacket.CommandType.Voltage)
            {
                var voltage = 2.56 * 2 * (packet.Data[1] + packet.Data[2] * 256) / 1024.0;
                if (voltage < 4.9)
                {
                    notifyIcon.ShowBalloonTip(10000, "Низкое напряжение", "Напряжение на устройстве №" + (packet.Data[0]) + ": " +
                    voltage + " вольт", ToolTipIcon.Warning);
                    //System.Media.SystemSounds.Beep.Play();
                }
            }
            if (packet.Command == (byte)ClunetPacket.CommandType.IntercomRing)
            {
                notifyIcon.ShowBalloonTip(10000, "Домофон", "Звонок в домофон!", ToolTipIcon.Warning);
                if (packet.Data.Length >= 1 && packet.Data[0] == 4)
                    showCamera();
            }
            if (packet.Command == (byte)ClunetPacket.CommandType.IntercomMessage)
            {
                notifyIcon.ShowBalloonTip(10000, "Домофон", packet.HumanReadableInfo(), ToolTipIcon.Warning);
            }
            if (packet.Command == (byte)ClunetPacket.CommandType.DoorLockInfo)
            {
                toolStripMenuItemDoorLockOpen.Checked = packet.Data[0] == 1;
                toolStripMenuItemDoorLockClose.Checked = packet.Data[0] == 2;
            }
            if (packet.Command == (byte)ClunetPacket.CommandType.WindowInfo)
            {
                toolStripMenuItemWindowOpenFull.Checked = packet.Data[0] == 0;
                toolStripMenuItemWindowOpen.Checked = packet.Data[0] == 1;
                toolStripMenuItemWindowClose.Checked = packet.Data[0] == 2;
            }

            var rowNum = dataGridViewLog.Rows.Add();
            var row = dataGridViewLog.Rows[rowNum];
            row.Cells["colDateTime"].Value = DateTime.Now;
            row.Cells["colSRC"].Value = packet.SrcAddress.ToString("X2") + " (" + GetDeviceName(packet.SrcAddress) + ")";
            row.Cells["colDST"].Value = packet.DstAddress.ToString("X2") + " (" + GetDeviceName(packet.DstAddress) + ")";
            row.Cells["colCMD"].Value = string.Format("{0:X2}", packet.Command) + " (" + ((ClunetPacket.CommandType)packet.Command).ToString() + ")";
            var data = new StringBuilder();
            foreach (var b in packet.Data) data.AppendFormat("{0:X2}", b);
            row.Cells["colData"].Value = data.ToString();
            row.Cells["colHumanText"].Value = packet.HumanReadableInfo();
            dataGridViewLog.FirstDisplayedScrollingRowIndex = rowNum;
            while (dataGridViewLog.RowCount > 100) dataGridViewLog.Rows.RemoveAt(0);

            // Состояние люстры
            if (packet.SrcAddress == 0x01 && (ClunetPacket.CommandType)packet.Command == ClunetPacket.CommandType.DevicePowerInfo && packet.Length > 0)
            {
                light0ToolStripMenuItem.Checked = packet.Data[0] == 0;
                light1ToolStripMenuItem.Checked = packet.Data[0] == 1;
                light2ToolStripMenuItem.Checked = packet.Data[0] == 2;
                light3ToolStripMenuItem.Checked = packet.Data[0] == 3;
            }

            // Режим домофона
            if (packet.SrcAddress == 0x07 && (ClunetPacket.CommandType)packet.Command == ClunetPacket.CommandType.IntercomModeInfo && packet.Length >= 2)
            {
                toolStripMenuItemCurrentMode0.Checked = packet.Data[0] == 0;
                toolStripMenuItemCurrentMode1.Checked = packet.Data[0] == 1;
                toolStripMenuItemCurrentMode2.Checked = packet.Data[0] == 2;
                toolStripMenuItemCurrentMode3.Checked = packet.Data[0] == 3;
                toolStripMenuItemCurrentMode4.Checked = packet.Data[0] == 4;
                toolStripMenuItemCurrentMode5.Checked = packet.Data[0] == 5;
                toolStripMenuItemTempMode0.Checked = packet.Data[1] == 0;
                toolStripMenuItemTempMode1.Checked = packet.Data[1] == 1;
                toolStripMenuItemTempMode2.Checked = packet.Data[1] == 2;
                toolStripMenuItemTempMode3.Checked = packet.Data[1] == 3;
                toolStripMenuItemTempMode4.Checked = packet.Data[1] == 4;
                toolStripMenuItemTempMode5.Checked = packet.Data[1] == 5;
            }
        }

        private delegate void CopyToClipboardFromThread(string text);
        void CopyToClipboard(string text)
        {
            if (InvokeRequired)
            {
                Invoke(new CopyToClipboardFromThread(CopyToClipboard), new object[] { text });
                return;
            }
            if (copyRcCodesToolStripMenuItem.Checked)
            {
                Clipboard.SetText(text);
                System.Media.SystemSounds.Hand.Play();
            }
        }
        void clunetConnection_OnPacketReceived(ClunetPacket packet)
        {
            switch ((ClunetPacket.CommandType)packet.Command)
            {
                case ClunetPacket.CommandType.DiscoveryResponse:
                    deviceNames[packet.SrcAddress] = packet.DataAsString;
                    break;
                case ClunetPacket.CommandType.Temperature:
                    temperature[packet.DataAsHex.Substring(0, 14)] = packet.GetTemperature();
                    break;
                case ClunetPacket.CommandType.RcButtonPressed:
                    UInt32 code = (UInt32)(packet.Data[1] + packet.Data[2] * 0x100 + packet.Data[3] * 0x10000 + packet.Data[4] * 0x1000000);
                    const string remoteName = "CLUNET_RC";
                    string buttonName;
                    if (packet.Data[0] % 2 == 1)
                    {
                        buttonName = string.Format("button:{1:X8},type:{0:X2}", packet.Data[0], code);
                        winLircServer.SendButtonCodeFirst(code, buttonName, remoteName);
                        CopyToClipboard(buttonName);
                    }
                    else
                    {
                        buttonName = string.Format("button:{1:X8},type:{0:X2}", packet.Data[0] - 1, code);
                        winLircServer.SendButtonCodeRepeat(code, buttonName, remoteName);
                    }
                    break;
                case ClunetPacket.CommandType.Motion:
                    noMotionTime = 0;
                    break;
                case ClunetPacket.CommandType.DoorBell:
                    doorBell();
                    break;
            }
            AddPacketToLog(packet);
        }

        void StartDiscovery()
        {
            clunetConnection.Send(new ClunetPacket(ClunetPacket.PacketPriority.Command, LocalAddress, 0xFF, ClunetPacket.CommandType.Discovery));
        }
        void SendPing()
        {
            var pingData = new byte[64];
            for (byte c = 0; c < pingData.Length; c++) pingData[c] = c;
            clunetConnection.Send(new ClunetPacket(ClunetPacket.PacketPriority.Command, LocalAddress, 0xFF, ClunetPacket.CommandType.Ping, pingData));
        }
        void RequestPowerInfo()
        {
            clunetConnection.Send(new ClunetPacket(ClunetPacket.PacketPriority.Command, LocalAddress, 1, ClunetPacket.CommandType.DevicePowerInfoRequest));
        }

        void RequestIntercomMode()
        {
            clunetConnection.Send(new ClunetPacket(ClunetPacket.PacketPriority.Command, LocalAddress, 7, ClunetPacket.CommandType.IntercomModeRequest));
        }

        void RequestDoorLockState()
        {
            clunetConnection.Send(new ClunetPacket(ClunetPacket.PacketPriority.Command, LocalAddress, 0x0A, ClunetPacket.CommandType.DoorLockInfoRequest));
        }

        void RequestWindowState()
        {
            clunetConnection.Send(new ClunetPacket(ClunetPacket.PacketPriority.Command, LocalAddress, 0x0C, ClunetPacket.CommandType.WindowInfoRequest));
        }

        string GetDeviceName(byte address)
        {
            if (address == 0xFF) return "широковещательный";
            string result = "";
            deviceNames.TryGetValue(address, out result);
            return result;
        }

        private void buttonDiscovery_Click(object sender, EventArgs e)
        {
            try
            {
                StartDiscovery();
            }
            catch (Exception ex)
            {
                MessageBox.Show(this, ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void buttonPing_Click(object sender, EventArgs e)
        {
            try
            {
                SendPing();
            }
            catch (Exception ex)
            {
                MessageBox.Show(this, ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void buttonSend_Click(object sender, EventArgs e)
        {
            try
            {
                var priority = (ClunetPacket.PacketPriority)(comboBoxPriority.SelectedIndex + 1);
                var device = byte.Parse(comboBoxDevices.Text.Substring(0, 2), NumberStyles.AllowHexSpecifier);
                var command = byte.Parse(comboBoxCommands.Text.Substring(0, 2), NumberStyles.AllowHexSpecifier);
                var data = new byte[textBoxData.Text.Length / 2];
                for (int i = 0; i < data.Length; i++)
                {
                    data[i] = byte.Parse(textBoxData.Text.Substring(i * 2, 2), NumberStyles.AllowHexSpecifier);
                }
                clunetConnection.Send(new ClunetPacket(priority, LocalAddress, device, command, data));
            }
            catch (Exception ex)
            {
                MessageBox.Show(this, ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void ClunetConsoleForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (e.CloseReason == CloseReason.UserClosing)
            {
                e.Cancel = true;
                Hide();
            }
            else
            {
                Exit();
            }
        }

        private void Exit()
        {
            notifyIcon.Visible = false;
            clunetConnection.Dispose();
            winLircServer.Dispose();
            //speechCapture.StopCapture();000
            Application.ExitThread();
        }

        private void notifyIcon_MouseDoubleClick(object sender, MouseEventArgs e)
        {
            this.Show();
            this.WindowState = FormWindowState.Normal;
            this.Activate();
        }

        private void showConsoleToolStripMenuItem_Click(object sender, EventArgs e)
        {
            this.Show();
            this.WindowState = FormWindowState.Normal;
            this.Activate();
        }

        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (MessageBox.Show(this, "Точно?", "Выйти?", MessageBoxButtons.YesNo, MessageBoxIcon.Warning, MessageBoxDefaultButton.Button1) == System.Windows.Forms.DialogResult.Yes)
            {
                Exit();
            }
        }

        private void remotesToolStripMenuItem_Click(object sender, EventArgs e)
        {
            philipsForm.Show();
            philipsForm.WindowState = FormWindowState.Normal;
            philipsForm.Activate();
            yamahaForm.Show();
            yamahaForm.WindowState = FormWindowState.Normal;
            yamahaForm.Activate();
            engyForm.Show();
            engyForm.WindowState = FormWindowState.Normal;
            engyForm.Activate();
            laserForm.Show();
            laserForm.WindowState = FormWindowState.Normal;
            laserForm.Activate();
        }

        private void ShowTemperatureBalloon()
        {
            var text = new StringBuilder();
            double temp;
            if (temperature.TryGetValue("28FF62CF131400", out temp))
                text.AppendFormat("На улице: {0} °C\r\n", temp);
            if (temperature.TryGetValue("28DC3C35040000", out temp))
                text.AppendFormat("На балконе: {0} °C\r\n", temp);
            if (temperature.TryGetValue("28F14D35040000", out temp))
                text.AppendFormat("В комнате у окна: {0} °C\r\n", temp);
            if (temperature.TryGetValue("287FA263030000", out temp))
                text.AppendFormat("В комнате у двери: {0} °C\r\n", temp);
            if (temperature.TryGetValue("28C88F63030000", out temp))
                text.AppendFormat("В коридоре: {0} °C\r\n", temp);
            if (temperature.TryGetValue("284E6335040000", out temp))
                text.AppendFormat("Радиатора отопления: {0} °C\r\n", temp);
            if (text.Length == 0) text.Append("Температура ещё не получена");
            notifyIcon.ShowBalloonTip(3000, "Температура", text.ToString(), ToolTipIcon.None);
        }

        private void showTemperatureToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ShowTemperatureBalloon();
        }

        private void doorBell()
        {
            if (InvokeRequired)
            {
                Invoke(new Action(doorBell));
                return;
            }
            notifyIcon.ShowBalloonTip(3000, "Звонок в дверь", "Дзинь!", ToolTipIcon.Info);
            showCamera();
        }

        CameraForm cameraForm = new CameraForm();
        private void showCamera()
        {
            const string cameraUrl = "http://cameras.clusterrr.com/?camera=4&pda&username=system&password=system";
            //try
            //{
            cameraForm.Left = 10;// SystemInformation.VirtualScreen.Width - cameraForm.Width - 10;
            cameraForm.Top = 10; //SystemInformation.VirtualScreen.Height - cameraForm.Height - 10;
            cameraForm.Show();
            cameraForm.BringToFront();
            cameraForm.webBrowser.Url = new Uri(cameraUrl);

            //}
            //catch
            //{
            //    cameraForm = new CameraForm();
            //    cameraForm.Show();
            //    cameraForm.BringToFront();
            //}
        }

        private void lightToolStripMenuItem_Click(object sender, EventArgs e)
        {
            try
            {
                clunetConnection.Send(new ClunetPacket(ClunetPacket.PacketPriority.Command, LocalAddress, 0x01, ClunetPacket.CommandType.DevicePowerCommand, new byte[] { byte.Parse((string)((ToolStripMenuItem)sender).Tag) }));
            }
            catch (Exception ex)
            {
                MessageBox.Show(this, ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void timerMouseCageCheck_Tick(object sender, EventArgs e)
        {
            if (clunetConnection.Connected)
            {
                /*
                for (int wheel = 1; wheel < lastWheelInfo.Length; wheel++)
                {
                    lastWheelInfo[wheel]++;
                    var howLong = lastWheelInfo[wheel];
                    if (howLong >= 10)
                    {
                        notifyIcon.ShowBalloonTip(10000, "Нет связи с колесом", "Нет связи с колесом №" + (wheel + 1) + " уже " +
                        howLong + " минут", ToolTipIcon.Warning);
                        System.Media.SystemSounds.Beep.Play();
                    }
                }
                 */
            }
        }

        private void timerNoMotion_Tick(object sender, EventArgs e)
        {
            noMotionTime++;
            try
            {
                File.WriteAllText(@"D:\nomotion.txt", noMotionTime.ToString());
            }
            catch { }
        }

        void speechCapture_OnSpeechRecognized(string[] result)
        {
            var commands = new Dictionary<string, ClunetPacket>();
            commands["^(вика )?(включить|включи) свет$"] = new ClunetPacket(ClunetPacket.PacketPriority.Command, LocalAddress, 0x01, ClunetPacket.CommandType.DevicePowerCommand, new byte[] { 0x03 });
            commands["^(вика )?(выключить|выключи) свет$"] = new ClunetPacket(ClunetPacket.PacketPriority.Command, LocalAddress, 0x01, ClunetPacket.CommandType.DevicePowerCommand, new byte[] { 0x00 });
            commands["^(вика )?одна лампа$"] = new ClunetPacket(ClunetPacket.PacketPriority.Command, LocalAddress, 0x01, ClunetPacket.CommandType.DevicePowerCommand, new byte[] { 0x01 });
            commands["^(вика )?две лампы$"] = new ClunetPacket(ClunetPacket.PacketPriority.Command, LocalAddress, 0x01, ClunetPacket.CommandType.DevicePowerCommand, new byte[] { 0x02 });
            commands["^(вика )?три лампы$"] = new ClunetPacket(ClunetPacket.PacketPriority.Command, LocalAddress, 0x01, ClunetPacket.CommandType.DevicePowerCommand, new byte[] { 0x03 });
            commands["^(вика )?(включить|выключить|включи|выключи) (лазер|пушку)$"] = new ClunetPacket(ClunetPacket.PacketPriority.Command, LocalAddress, 0x04, ClunetPacket.CommandType.RcButtonSend, new byte[] { 0x01, 0x86, 0x6B, 0x12, 0xED });
            foreach (var phrase in result)
                foreach (var command in commands.Keys)
                {
                    Regex r = new Regex(command);
                    var mc = r.Matches(phrase);
                    if (mc.Count > 0)
                    {
                        clunetConnection.Send(commands[command]);
                        return;
                    }
                }
            var text = String.Join("\r\n", result);
            notifyIcon.ShowBalloonTip(3000, "Не поняла", text, ToolTipIcon.Warning);
        }

        void speechCapture_OnSpeechRecognizeFailed()
        {
            notifyIcon.ShowBalloonTip(3000, "Не поняла", "Ты что-то сказал или мне послышалось?", ToolTipIcon.Warning);
        }

        void speechCapture_OnCaptureEror(Exception exception)
        {
            notifyIcon.ShowBalloonTip(3000, "Распознавание речи", "Распознавание речи перезапущено", ToolTipIcon.Warning);
        }

        private void перезапуститьРаспознаваниеРечиToolStripMenuItem_Click(object sender, EventArgs e)
        {
            //speechCapture.StartCapture();
        }

        private void toolStripMenuItemCurrentMode0_Click(object sender, EventArgs e)
        {
            try
            {
                clunetConnection.Send(new ClunetPacket(ClunetPacket.PacketPriority.Command, LocalAddress, 0x07, ClunetPacket.CommandType.IntercomModeSet, new byte[] { byte.Parse((string)((ToolStripMenuItem)sender).Tag) }));
            }
            catch (Exception ex)
            {
                MessageBox.Show(this, ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void toolStripMenuItemTempMode0_Click(object sender, EventArgs e)
        {
            try
            {
                clunetConnection.Send(new ClunetPacket(ClunetPacket.PacketPriority.Command, LocalAddress, 0x07, ClunetPacket.CommandType.IntercomModeSet, new byte[] { 0xFF, byte.Parse((string)((ToolStripMenuItem)sender).Tag) }));
            }
            catch (Exception ex)
            {
                MessageBox.Show(this, ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void toolStripMenuItemDoorLockOpen_Click(object sender, EventArgs e)
        {
            try
            {
                clunetConnection.Send(new ClunetPacket(ClunetPacket.PacketPriority.Command, LocalAddress, 0x0A, ClunetPacket.CommandType.DoorLockCommand, new byte[] { 0x01 }));
            }
            catch (Exception ex)
            {
                MessageBox.Show(this, ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void toolStripMenuItemDoorLockClose_Click(object sender, EventArgs e)
        {
            try
            {
                clunetConnection.Send(new ClunetPacket(ClunetPacket.PacketPriority.Command, LocalAddress, 0x0A, ClunetPacket.CommandType.DoorLockCommand, new byte[] { 0x02 }));
            }
            catch (Exception ex)
            {
                MessageBox.Show(this, ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void toolStripMenuItemWindowOpenFull_Click(object sender, EventArgs e)
        {
            try
            {
                clunetConnection.Send(new ClunetPacket(ClunetPacket.PacketPriority.Command, LocalAddress, 0x0C, ClunetPacket.CommandType.WindowCommand, new byte[] { 0x00 }));
            }
            catch (Exception ex)
            {
                MessageBox.Show(this, ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void toolStripMenuItemWindowOpen_Click(object sender, EventArgs e)
        {
            try
            {
                clunetConnection.Send(new ClunetPacket(ClunetPacket.PacketPriority.Command, LocalAddress, 0x0C, ClunetPacket.CommandType.WindowCommand, new byte[] { 0x01 }));
            }
            catch (Exception ex)
            {
                MessageBox.Show(this, ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void toolStripMenuItemWindowClose_Click(object sender, EventArgs e)
        {
            try
            {
                clunetConnection.Send(new ClunetPacket(ClunetPacket.PacketPriority.Command, LocalAddress, 0x0C, ClunetPacket.CommandType.WindowCommand, new byte[] { 0x02 }));
            }
            catch (Exception ex)
            {
                MessageBox.Show(this, ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void toolStripMenuItemWindowOpen5min_Click(object sender, EventArgs e)
        {
            try
            {
                clunetConnection.Send(new ClunetPacket(ClunetPacket.PacketPriority.Command, LocalAddress, 0x0C, ClunetPacket.CommandType.SetTimer, new byte[] { 0x2C, 0x01 }));
            }
            catch (Exception ex)
            {
                MessageBox.Show(this, ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void toolStripMenuItemWindowOpen10min_Click(object sender, EventArgs e)
        {
            try
            {
                clunetConnection.Send(new ClunetPacket(ClunetPacket.PacketPriority.Command, LocalAddress, 0x0C, ClunetPacket.CommandType.SetTimer, new byte[] { 0x58, 0x02 }));
            }
            catch (Exception ex)
            {
                MessageBox.Show(this, ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void toolStripMenuItemWindowOpen15min_Click(object sender, EventArgs e)
        {
            try
            {
                clunetConnection.Send(new ClunetPacket(ClunetPacket.PacketPriority.Command, LocalAddress, 0x0C, ClunetPacket.CommandType.SetTimer, new byte[] { 0x84, 0x03 }));
            }
            catch (Exception ex)
            {
                MessageBox.Show(this, ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void toolStripMenuItemWindowOpen30min_Click(object sender, EventArgs e)
        {
            try
            {
                clunetConnection.Send(new ClunetPacket(ClunetPacket.PacketPriority.Command, LocalAddress, 0x0C, ClunetPacket.CommandType.SetTimer, new byte[] { 0x08, 0x07 }));
            }
            catch (Exception ex)
            {
                MessageBox.Show(this, ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void toolStripMenuItemWindowOpen60min_Click(object sender, EventArgs e)
        {
            try
            {
                clunetConnection.Send(new ClunetPacket(ClunetPacket.PacketPriority.Command, LocalAddress, 0x0C, ClunetPacket.CommandType.SetTimer, new byte[] { 0x10, 0x0E }));
            }
            catch (Exception ex)
            {
                MessageBox.Show(this, ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void cameraToolStripMenuItem_Click(object sender, EventArgs e)
        {
            showCamera();
        }

    }
}
