﻿using System;
using System.Collections.Generic;
using System.IO.Ports;
using System.Linq;
using System.Net;
using System.Net.Sockets;
using System.Text;
using System.Threading;

namespace Cluster.WinLIRC
{
    class WinLircServer : IDisposable
    {
        TcpListener tcpListener;
        Thread tcpListenerThread;
        List<Thread> connectionThreads = new List<Thread>();
        List<Socket> sockets = new List<Socket>();
        private bool disposing = false;
        byte repeats = 0;
        UInt32 lastCode = 0;

        public WinLircServer(int tcpPort = 8765)
        {
            try
            {
                tcpListener = new TcpListener(IPAddress.Any, tcpPort);
                tcpListener.Start();
                //Console.WriteLine("TCP порт " + tcpPort + " открыт и слушается.");
                tcpListenerThread = new Thread(ListenTCP);
                tcpListenerThread.Start();
            }
            catch
            {
                //Console.WriteLine("Critical exception: " + ex.Message);
            }
        }

        private void ListenTCP()
        {
            try
            {
                while (true)
                {
                    var socket = tcpListener.AcceptSocket();
                    var thread = new Thread(ConnectionThread);
                    connectionThreads.Add(thread);
                    thread.Start(socket);
                }
            }
            catch (ThreadAbortException)
            {
            }
            catch (Exception ex)
            {
                if (!disposing)
                    Console.WriteLine("Exception: " + ex.ToString() + ", " + ex.Message);
            }
            finally
            {
                try
                {
                    tcpListener.Stop();
                }
                catch { }
            }
        }

        private void ConnectionThread(object o)
        {
            var socket = (Socket)o;
            try
            {
                Console.WriteLine("Подключился: " + socket.RemoteEndPoint.ToString());
                sockets.Add(socket);
                var buffer = new byte[1]; // лол. да
                StringBuilder line = new StringBuilder();
                while (true)
                {
                    int len = socket.Receive(buffer);
                    if (len == 0)
                    {
                        Console.WriteLine("Отключился: " + socket.RemoteEndPoint.ToString());
                        return;
                    }
                }
            }
            catch (ThreadAbortException)
            {
                try
                {
                    socket.Close();
                }
                catch { }
            }
            catch (SocketException) { }
            catch (Exception ex)
            {
                if (!disposing)
                    Console.WriteLine("Exception: " + ex.ToString() + ", " + ex.Message);
            }
            finally
            {
                try
                {
                    socket.Close();
                }
                catch { }
                sockets.Remove(socket);
            }
        }

        public void SendButtonCode(UInt32 buttonCode, string buttonName, string rcName, byte repeatCount = 0)
        {
            SendToNet(string.Format("00000000{0:x8} {1:x2} {2} {3}\n", buttonCode, repeatCount, buttonName, rcName));
            lastCode = buttonCode;
        }

        public void SendButtonCodeFirst(UInt32 buttonCode, string buttonName, string rcName)
        {
           repeats = 0;
           SendButtonCode(buttonCode, buttonName, rcName, 0);
        }

        public void SendButtonCodeRepeat(UInt32 buttonCode, string buttonName, string rcName)
        {
            if (buttonCode != lastCode) repeats = 0;
            repeats++;
            SendButtonCode(buttonCode, buttonName, rcName, repeats);
        }

        private void SendToNet(string line)
        {
            var data = Encoding.ASCII.GetBytes(line);
            foreach (var socket in sockets)
                socket.Send(data);
        }

        public void Dispose()
        {
            disposing = true;
            foreach (var socket in sockets)
            {
                socket.Close();
            }
            foreach (var thread in connectionThreads)
            {
                if (thread.ThreadState == ThreadState.Running)
                    thread.Abort();
            }
            tcpListener.Stop();
            if (tcpListenerThread.ThreadState == ThreadState.Running)
                tcpListenerThread.Abort();
        }
    }
}
